import React from 'react';
import { defaultProps } from '../../Animation/props/defaultProps';
import { propTypes } from '../../Animation/props/propTypes';
import { CSSTransition } from 'react-transition-group';
import style from '../../common/transition.module.css';
export default function Animation(props) {
  let {
    name,
    enterDuration,
    exitDuration,
    children,
    isActive,
    unmountOnExit,
    onEntered = () => {},
    onExit = () => {},
    delayClassName = null
  } = props;
  const animation = {
    zoomIn: {
      enter: style.zoomInEnter,
      enterActive: style.zoomInEnterActive,
      enterDone: style.zoomInEnterActive,
      exit: style.zoomInLeave,
      exitActive: style.zoomInLeaveActive,
      exitDone: style.zoomInLeaveActive,
      appear: style.zoomInEnter,
      appearActive: style.zoomInEnterActive
    },
    scaleIn: {
      enter: style.scaleInEnter,
      enterActive: style.scaleInEnterActive,
      enterDone: style.scaleInEnterActive,
      exit: style.scaleInLeave,
      exitActive: style.scaleInLeaveActive,
      exitDone: style.scaleInLeaveActive,
      appear: style.scaleInEnter,
      appearActive: style.scaleInEnterActive
    },
    fadeIn: {
      enter: style.fadeInEnter,
      enterActive: style.fadeInEnterActive,
      enterDone: style.fadeInEnterActive,
      exit: style.fadeInLeave,
      exitActive: style.fadeInLeaveActive,
      exitDone: style.fadeInLeaveActive,
      appear: style.fadeInEnter,
      appearActive: style.fadeInEnterActive
    },
    slideLeft: {
      enter: style.slideLeftEnter,
      enterActive: style.slideLeftEnterActive,
      enterDone: style.slideLeftEnterActive,
      exit: style.slideLeftLeave,
      exitActive: style.slideLeftLeaveActive,
      exitDone: style.slideLeftLeaveActive,
      appear: style.slideLeftEnter,
      appearActive: style.slideLeftEnterActive
    },
    slideDown: {
      enter: style.slideDownEnter,
      enterActive: style.slideDownEnterActive,
      enterDone: style.slideDownEnterActive,
      exit: style.slideDownLeave,
      exitActive: style.slideDownLeaveActive,
      exitDone: style.slideDownLeaveActive,
      appear: style.slideDownEnter,
      appearActive: style.slideDownEnterActive
    },
    skewIn: {
      enter: style.skewInEnter,
      enterActive: style.skewInEnterActive,
      enterDone: style.skewInEnterActive,
      exit: style.skewInLeave,
      exitActive: style.skewInLeaveActive,
      exitDone: style.skewInLeaveActive,
      appear: style.skewInEnter,
      appearActive: style.skewInEnterActive
    },
    none: {
      enter: style.noneEnter,
      enterActive: style.noneEnterActive,
      enterDone: style.noneEnterActive,
      exit: style.noneLeave,
      exitActive: style.noneLeaveActive,
      exitDone: style.noneLeaveActive,
      appear: style.noneEnter,
      appearActive: style.noneEnterActive
    },
    default: {
      enter: style.defaultEnter,
      enterActive: style.defaultEnterActive,
      enterDone: style.defaultEnterActive,
      exit: style.defaultLeave,
      exitActive: style.defaultLeaveActive,
      exitDone: style.defaultLeaveActive,
      appear: style.defaultEnter,
      appearActive: style.defaultEnterActive
    }
  };
  let newClassNames = delayClassName ? Object.assign({}, animation[name], {
    exit: `${delayClassName} ${animation[name].exit}`
  }) : animation[name];
  return /*#__PURE__*/React.createElement(CSSTransition, {
    classNames: newClassNames && newClassNames,
    in: isActive,
    appear: true,
    enter: true,
    exit: true,
    mountOnEnter: false,
    unmountOnExit: unmountOnExit,
    timeout: {
      enter: enterDuration,
      exit: exitDuration
    },
    onEntered: onEntered,
    onExit: onExit
  }, /*#__PURE__*/React.createElement(React.Fragment, null, children));
}
Animation.propTypes = propTypes;
Animation.defaultProps = defaultProps; // if (__DOCS__) {
//   Animation.docs = {
//     componentGroup: 'Animation',
//     folderName: 'Style Guide',
//     external: true,
//     description: ' '
//   };
// }