function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React, { useEffect, useRef, useLayoutEffect } from 'react';
import { defaultProps } from '../../AppContainer/props/defaultProps';
import { propTypes } from '../../AppContainer/props/propTypes';
import { Container, Box } from '../Layout';
import Tooltip from '../Tooltip/Tooltip';
import '../../common/basic.module.css';
import '@zohodesk/variables/assets/colorVariables.module.css';
import '@zohodesk/variables/assets/dotVariables.module.css';
import '@zohodesk/variables/assets/sizeVariables.module.css';
import '@zohodesk/variables/assets/fontsizeVariables.module.css';
import '@zohodesk/variables/lib/fontFamilyVariables.module.css';
import '@zohodesk/variables/assets/transitionVariables.module.css';
import '@zohodesk/variables/assets/no_transitionVariables.module.css';
import '../../common/a11y.module.css';
import '../../common/boxShadow.module.css';
import style from '../../AppContainer/AppContainer.module.css';
import { getLibraryConfig, setLibraryConfig } from '../../Provider/Config';
export default function AppContainer(props) {
  let {
    className,
    children,
    tagName,
    dataId,
    dataSelectorId,
    tooltipClass,
    tooltipParentClass,
    needTooltip,
    customProps,
    eleRef
  } = props;
  let {
    ContainerProps = {},
    TooltipProps = {},
    ExtraProps = {}
  } = customProps;
  const tooltipRef = useRef(null);
  const containerElement = useRef(null);
  const timer = useRef(null);
  const tooltipDebounce = useRef(getLibraryConfig('tooltipDebounce'));
  useLayoutEffect(() => {
    setLibraryConfig({
      getTooltipContainer: () => containerElement.current
    });
  }, []);

  function mouseOverDispatch(e) {
    if (tooltipRef.current) {
      tooltipRef.current.handleOver(e, containerElement.current);
    }
  }

  function removeTimeout() {
    if (timer.current) {
      timer.current = clearTimeout(timer.current);
    }
  }

  function handleOver(e) {
    if (timer.current) {
      timer.current = clearTimeout(timer.current);
    }

    timer.current = setTimeout(() => mouseOverDispatch(e), tooltipDebounce.current);
  }

  function getContainerRef(ref) {
    containerElement.current = ref;
    eleRef && eleRef(ref);
  }

  function setTooltipRef(ref) {
    tooltipRef.current = ref;
  }

  useEffect(() => {
    if (containerElement.current && needTooltip) {
      containerElement.current.addEventListener('mouseover', handleOver, false);
      containerElement.current.addEventListener('mouseout', removeTimeout, false);
      tooltipRef.current.observeElement();
    }
  }, []);
  useEffect(() => {
    return () => {
      if (containerElement.current && needTooltip) {
        containerElement.current.removeEventListener('mouseover', handleOver, false);
        containerElement.current.addEventListener('mouseout', removeTimeout, false);
        tooltipRef.current.unObserveElement();
        setLibraryConfig({
          getTooltipContainer: () => null
        });
      }
    };
  }, []);
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Container, _extends({
    className: `${style.container} ${className}`,
    dataId: dataId,
    dataSelectorId: dataSelectorId,
    tagName: tagName,
    eleRef: getContainerRef
  }, ContainerProps, ExtraProps), /*#__PURE__*/React.createElement(Box, {
    flexible: true
  }, children)), needTooltip ? /*#__PURE__*/React.createElement("div", _extends({}, ExtraProps, {
    className: `${style.container} ${style.tooltip} ${tooltipParentClass}`,
    "data-id": `${dataId}_tooltip`,
    "data-test-id": `${dataId}_tooltip`,
    "data-selector-id": `${dataSelectorId}_tooltip`
  }), /*#__PURE__*/React.createElement(Tooltip, _extends({
    ref: setTooltipRef,
    customClass: tooltipClass
  }, TooltipProps))) : null);
}
AppContainer.propTypes = propTypes;
AppContainer.defaultProps = defaultProps; // if (__DOCS__) {
//   AppContainer.docs = {
//     componentGroup: 'Template',
//     folderName: 'Style Guide'
//   };
// }