"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = Animation;

var _react = _interopRequireDefault(require("react"));

var _defaultProps = require("../../Animation/props/defaultProps");

var _propTypes = require("../../Animation/props/propTypes");

var _reactTransitionGroup = require("react-transition-group");

var _transitionModule = _interopRequireDefault(require("../../common/transition.module.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function Animation(props) {
  var name = props.name,
      enterDuration = props.enterDuration,
      exitDuration = props.exitDuration,
      children = props.children,
      isActive = props.isActive,
      unmountOnExit = props.unmountOnExit,
      _props$onEntered = props.onEntered,
      onEntered = _props$onEntered === void 0 ? function () {} : _props$onEntered,
      _props$onExit = props.onExit,
      onExit = _props$onExit === void 0 ? function () {} : _props$onExit,
      _props$delayClassName = props.delayClassName,
      delayClassName = _props$delayClassName === void 0 ? null : _props$delayClassName;
  var animation = {
    zoomIn: {
      enter: _transitionModule["default"].zoomInEnter,
      enterActive: _transitionModule["default"].zoomInEnterActive,
      enterDone: _transitionModule["default"].zoomInEnterActive,
      exit: _transitionModule["default"].zoomInLeave,
      exitActive: _transitionModule["default"].zoomInLeaveActive,
      exitDone: _transitionModule["default"].zoomInLeaveActive,
      appear: _transitionModule["default"].zoomInEnter,
      appearActive: _transitionModule["default"].zoomInEnterActive
    },
    scaleIn: {
      enter: _transitionModule["default"].scaleInEnter,
      enterActive: _transitionModule["default"].scaleInEnterActive,
      enterDone: _transitionModule["default"].scaleInEnterActive,
      exit: _transitionModule["default"].scaleInLeave,
      exitActive: _transitionModule["default"].scaleInLeaveActive,
      exitDone: _transitionModule["default"].scaleInLeaveActive,
      appear: _transitionModule["default"].scaleInEnter,
      appearActive: _transitionModule["default"].scaleInEnterActive
    },
    fadeIn: {
      enter: _transitionModule["default"].fadeInEnter,
      enterActive: _transitionModule["default"].fadeInEnterActive,
      enterDone: _transitionModule["default"].fadeInEnterActive,
      exit: _transitionModule["default"].fadeInLeave,
      exitActive: _transitionModule["default"].fadeInLeaveActive,
      exitDone: _transitionModule["default"].fadeInLeaveActive,
      appear: _transitionModule["default"].fadeInEnter,
      appearActive: _transitionModule["default"].fadeInEnterActive
    },
    slideLeft: {
      enter: _transitionModule["default"].slideLeftEnter,
      enterActive: _transitionModule["default"].slideLeftEnterActive,
      enterDone: _transitionModule["default"].slideLeftEnterActive,
      exit: _transitionModule["default"].slideLeftLeave,
      exitActive: _transitionModule["default"].slideLeftLeaveActive,
      exitDone: _transitionModule["default"].slideLeftLeaveActive,
      appear: _transitionModule["default"].slideLeftEnter,
      appearActive: _transitionModule["default"].slideLeftEnterActive
    },
    slideDown: {
      enter: _transitionModule["default"].slideDownEnter,
      enterActive: _transitionModule["default"].slideDownEnterActive,
      enterDone: _transitionModule["default"].slideDownEnterActive,
      exit: _transitionModule["default"].slideDownLeave,
      exitActive: _transitionModule["default"].slideDownLeaveActive,
      exitDone: _transitionModule["default"].slideDownLeaveActive,
      appear: _transitionModule["default"].slideDownEnter,
      appearActive: _transitionModule["default"].slideDownEnterActive
    },
    skewIn: {
      enter: _transitionModule["default"].skewInEnter,
      enterActive: _transitionModule["default"].skewInEnterActive,
      enterDone: _transitionModule["default"].skewInEnterActive,
      exit: _transitionModule["default"].skewInLeave,
      exitActive: _transitionModule["default"].skewInLeaveActive,
      exitDone: _transitionModule["default"].skewInLeaveActive,
      appear: _transitionModule["default"].skewInEnter,
      appearActive: _transitionModule["default"].skewInEnterActive
    },
    none: {
      enter: _transitionModule["default"].noneEnter,
      enterActive: _transitionModule["default"].noneEnterActive,
      enterDone: _transitionModule["default"].noneEnterActive,
      exit: _transitionModule["default"].noneLeave,
      exitActive: _transitionModule["default"].noneLeaveActive,
      exitDone: _transitionModule["default"].noneLeaveActive,
      appear: _transitionModule["default"].noneEnter,
      appearActive: _transitionModule["default"].noneEnterActive
    },
    "default": {
      enter: _transitionModule["default"].defaultEnter,
      enterActive: _transitionModule["default"].defaultEnterActive,
      enterDone: _transitionModule["default"].defaultEnterActive,
      exit: _transitionModule["default"].defaultLeave,
      exitActive: _transitionModule["default"].defaultLeaveActive,
      exitDone: _transitionModule["default"].defaultLeaveActive,
      appear: _transitionModule["default"].defaultEnter,
      appearActive: _transitionModule["default"].defaultEnterActive
    }
  };
  var newClassNames = delayClassName ? Object.assign({}, animation[name], {
    exit: "".concat(delayClassName, " ").concat(animation[name].exit)
  }) : animation[name];
  return /*#__PURE__*/_react["default"].createElement(_reactTransitionGroup.CSSTransition, {
    classNames: newClassNames && newClassNames,
    "in": isActive,
    appear: true,
    enter: true,
    exit: true,
    mountOnEnter: false,
    unmountOnExit: unmountOnExit,
    timeout: {
      enter: enterDuration,
      exit: exitDuration
    },
    onEntered: onEntered,
    onExit: onExit
  }, /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, children));
}

Animation.propTypes = _propTypes.propTypes;
Animation.defaultProps = _defaultProps.defaultProps; // if (__DOCS__) {
//   Animation.docs = {
//     componentGroup: 'Animation',
//     folderName: 'Style Guide',
//     external: true,
//     description: ' '
//   };
// }